(function () {
	function extendScheduler(scheduler) {
		if (typeof (dataProcessor) !== "undefined") {

			dataProcessor.prototype.live_updates = function (url) {
				if (typeof Faye === "undefined") return;

				var self = this;
				this.attachEvent("onAfterUpdate", function (sid, action, tid, tag) {

					var item;
					if (self.obj.exists(tid))
						item = self.obj.item(tid);
					else if (self.obj.exists(sid))
						item = self.obj.item(sid);
					else
						item = {};

					if (typeof (item.$selected) !== 'undefined') delete item.$selected;
					if (typeof (item.$template) !== 'undefined') delete item.$template;

					self.callEvent("onLocalUpdate", [{ sid: sid, tid: tid, status: action, data: item }]);

				});

				var client = new Faye.Client(url);
				client.subscribe('/update', function (message) {
					self.applyChanges(JSON.parse(message));
				});

				this.attachEvent("onLocalUpdate", function (data) {
					client.publish('/broadcast', JSON.stringify(data));
				});

			};

			dataProcessor.prototype.applyChanges = function (update) {
				var self = this;
				var sid = update.sid;
				var tid = update.tid;
				var status = update.status;
				var data = update.data;
				// prevent selection loosing
				if (self.obj.isSelected(sid)) data.$selected = true;
				switch (status) {
					case 'updated':
					case 'update':
					case 'inserted':
					case 'insert':
						if (self.obj.exists(sid)) {
							if (self.obj.isLUEdit(data) === sid) {
								self.obj.stopEditBefore();
							};
							self.ignore(function () {
								self.obj.update(sid, data);
								if (sid !== tid) self.obj.changeId(sid, tid);
							});
						} else {
							data.id = tid;
							self.ignore(function () {
								self.obj.add(data);
							});
						}
						break;
					case 'deleted':
					case 'delete':
						self.ignore(function () {
							var isExists = self.obj.exists(sid);
							if (isExists) {
								self.obj.setUserData(sid, '!nativeeditor_status', 'true_deleted');
								self.obj.stopEditBefore();
							}

							self.obj.remove(sid, data);

							if (isExists) {
								if (self.obj.isLUEdit(data) === sid) {
									// prevent loosing not-saved data
									self.obj.preventLUCollision(data);
									if (self.obj.callEvent("onLiveUpdateCollision", [sid, tid, status, data]) === false) {
										// we have to close editor here without saving
										self.obj.stopEditAfter();
									}
								}
							}
						});
						break;
				}
			};



		}

		if (typeof (scheduler) !== "undefined") {
			var dateFormat = "%Y-%m-%d %H:%i";
			var dateToStr = scheduler.date.date_to_str(dateFormat);
			var strToDate = scheduler.date.str_to_date(dateFormat);
			scheduler.item = function (id) {
				var event = this.getEvent(id);
				if (!event) return {};
				var data = {};
				for (var i in event)
					data[i] = event[i];

				data.start_date = dateToStr(event.start_date);
				data.end_date = dateToStr(event.end_date);
				return data;
			};
			scheduler.update = function (id, data) {
				var event = this.getEvent(id);
				for (var i in data)
					if (i != 'start_date' && i != 'end_date')
						event[i] = data[i];

				event.start_date = strToDate(data.start_date);
				event.end_date = strToDate(data.end_date);
				this.updateEvent(id);
				this.callEvent("onEventChanged", [id, event]);
			};
			scheduler.remove = function (id, data) {
				if (this.exists(id)) {

					var ev = this.getEvent(id);
					if (this._get_rec_markers) {
						if (ev.rec_type)
							this._roll_back_dates(ev);



						var sub = this._get_rec_markers(id);
						for (var i in sub) {
							if (sub.hasOwnProperty(i)) {
								id = sub[i].id;
								if (this.getEvent(id))
									this.deleteEvent(id, true);
							}
						}
					}

					this.deleteEvent(id, true);
				} else {
					if (data && data.event_pid) {
						scheduler.add(data);
					}

				}
			};

			scheduler.exists = function (id) {
				var event = this.getEvent(id);
				return event ? true : false;
			};

			scheduler.add = function (data) {
				data.start_date = strToDate(data.start_date);
				data.end_date = strToDate(data.end_date);
				var id = this.addEvent(data);
				if (this._is_modified_occurence && this._is_modified_occurence(data))
					this.setCurrentView();
				return id;
			};

			scheduler.changeId = function (old_id, new_id) {
				return this.changeEventId(old_id, new_id);
			};

			scheduler.stopEditBefore = function () { };

			scheduler.stopEditAfter = function () {
				this.endLightbox(false, this._lightbox);
			};

			scheduler.preventLUCollision = function (data) {
				this._new_event = this._lightbox_id;
				data.id = this._lightbox_id;
				this._events[this._lightbox_id] = data;
			};

			scheduler.isLUEdit = function (data) {
				if (this._lightbox_id)
					return this._lightbox_id;
				return null;
			};

			scheduler.isSelected = function (id) { return false; };
		}

	};

	if (window.Scheduler) {
		window.Scheduler.plugin(extendScheduler);
	} else {
		extendScheduler(window.scheduler);
	}
})();